//----------------------------------------------------------------------------
// user_logic.vhd - module
//----------------------------------------------------------------------------
//
// ***************************************************************************
// ** Copyright (c) 1995-2008 Xilinx, Inc.  All rights reserved.            **
// **                                                                       **
// ** Xilinx, Inc.                                                          **
// ** XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"         **
// ** AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND       **
// ** SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,        **
// ** OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,        **
// ** APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION           **
// ** THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,     **
// ** AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE      **
// ** FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY              **
// ** WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE               **
// ** IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR        **
// ** REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF       **
// ** INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS       **
// ** FOR A PARTICULAR PURPOSE.                                             **
// **                                                                       **
// ***************************************************************************
//
//----------------------------------------------------------------------------
// Filename:          user_logic.vhd
// Version:           1.00.a
// Description:       User logic module.
// Date:              Tue Mar 03 21:47:14 2009 (by Create and Import Peripheral Wizard)
// Verilog Standard:  Verilog-2001
//----------------------------------------------------------------------------


module user_logic
(
  // -- ADD USER PORTS BELOW THIS LINE ---------------
  
	//Video Capture lines, used for VDEC control
	RESET_VDEC1_Z,  
	VDEC1_OE_Z,
	VDEC1_PWRDN_Z,
	system_dcm_locked,
	YCrCb_in, 				//raw video input
	LLC_CLOCK, 				// vdec clock
	
	// mem_to_ip control lines
	buf_address, 			//what pixel we want to read		
	buf_read_data, 		// the pixel that is read
	buf_pixel_count, 		// the current pixel that is coming from input
	buf_line_count, 		// the current line that is coming from input
	buf_read_enable, 		// whether we enable the line buffer 0 for reading
	buf_write_enable, 	// whether we enable the line buffer 0 for writing
	
  // -- ADD USER PORTS ABOVE THIS LINE ---------------

  // -- DO NOT EDIT BELOW THIS LINE ------------------
  // -- Bus protocol ports, do not add to or delete 
  Bus2IP_Clk,                     // Bus to IP clock
  Bus2IP_Reset,                   // Bus to IP reset
  Bus2IP_Data,                    // Bus to IP data bus
  Bus2IP_BE,                      // Bus to IP byte enables
  Bus2IP_RdCE,                    // Bus to IP read chip enable
  Bus2IP_WrCE,                    // Bus to IP write chip enable
  IP2Bus_Data,                    // IP to Bus data bus
  IP2Bus_RdAck,                   // IP to Bus read transfer acknowledgement
  IP2Bus_WrAck,                   // IP to Bus write transfer acknowledgement
  IP2Bus_Error,                   // IP to Bus error response
  IP2Bus_MstRd_Req,               // IP to Bus master read request
  IP2Bus_MstWr_Req,               // IP to Bus master write request
  IP2Bus_Mst_Addr,                // IP to Bus master address bus
  IP2Bus_Mst_BE,                  // IP to Bus master byte enables
  IP2Bus_Mst_Length,              // IP to Bus master transfer length
  IP2Bus_Mst_Type,                // IP to Bus master transfer type
  IP2Bus_Mst_Lock,                // IP to Bus master lock
  IP2Bus_Mst_Reset,               // IP to Bus master reset
  Bus2IP_Mst_CmdAck,              // Bus to IP master command acknowledgement
  Bus2IP_Mst_Cmplt,               // Bus to IP master transfer completion
  Bus2IP_Mst_Error,               // Bus to IP master error response
  Bus2IP_Mst_Rearbitrate,         // Bus to IP master re-arbitrate
  Bus2IP_Mst_Cmd_Timeout,         // Bus to IP master command timeout
  Bus2IP_MstRd_d,                 // Bus to IP master read data bus
  Bus2IP_MstRd_rem,               // Bus to IP master read remainder
  Bus2IP_MstRd_sof_n,             // Bus to IP master read start of frame
  Bus2IP_MstRd_eof_n,             // Bus to IP master read end of frame
  Bus2IP_MstRd_src_rdy_n,         // Bus to IP master read source ready
  Bus2IP_MstRd_src_dsc_n,         // Bus to IP master read source discontinue
  IP2Bus_MstRd_dst_rdy_n,         // IP to Bus master read destination ready
  IP2Bus_MstRd_dst_dsc_n,         // IP to Bus master read destination discontinue
  IP2Bus_MstWr_d,                 // IP to Bus master write data bus
  IP2Bus_MstWr_rem,               // IP to Bus master write remainder
  IP2Bus_MstWr_sof_n,             // IP to Bus master write start of frame
  IP2Bus_MstWr_eof_n,             // IP to Bus master write end of frame
  IP2Bus_MstWr_src_rdy_n,         // IP to Bus master write source ready
  IP2Bus_MstWr_src_dsc_n,         // IP to Bus master write source discontinue
  Bus2IP_MstWr_dst_rdy_n,         // Bus to IP master write destination ready
  Bus2IP_MstWr_dst_dsc_n          // Bus to IP master write destination discontinue
  // -- DO NOT EDIT ABOVE THIS LINE ------------------
); // user_logic

// -- ADD USER PARAMETERS BELOW THIS LINE ------------
parameter C_LIVE_FRAME_ADDR             = 32'h90000000;
parameter C_REF_FRAME_ADDR              = 32'h90400000;
// -- ADD USER PARAMETERS ABOVE THIS LINE ------------

// -- DO NOT EDIT BELOW THIS LINE --------------------
// -- Bus protocol parameters, do not add to or delete
parameter C_SLV_DWIDTH                   = 32;
parameter C_MST_AWIDTH                   = 32;
parameter C_MST_DWIDTH                   = 32;
parameter C_NUM_REG                      = 9;
// -- DO NOT EDIT ABOVE THIS LINE --------------------

// -- ADD USER PORTS BELOW THIS LINE -----------------
input 	[9:2] 	YCrCb_in;          		 // YCrCb digital video data from VDEC1 daughter board
input 				LLC_CLOCK;               // Line Locked Clock (27MHz) from VDEC1 daughter board
output 				RESET_VDEC1_Z;           // RESET the video decoder on the VDEC1 daughter board
output 				VDEC1_OE_Z;              // ENABLE the video decoder outputs on the VDEC1 daughter board
output 				VDEC1_PWRDN_Z;           // POWER DOWN the video decoder on the VDEC1 daughter board
input 				system_dcm_locked;       // reset from top level module

// mem_to_ip <-> ip_to_mem signals
output 	[10:0]	buf_address; 				
input 	[0:31]	buf_read_data; 
output  	[10:0]   buf_pixel_count; 
output   [9:0] 	buf_line_count; 
output				buf_read_enable;		
output				buf_write_enable; 
// -- ADD USER PORTS ABOVE THIS LINE -----------------

// -- DO NOT EDIT BELOW THIS LINE --------------------
// -- Bus protocol ports, do not add to or delete
input                                     Bus2IP_Clk;
input                                     Bus2IP_Reset;
input      [0 : C_SLV_DWIDTH-1]           Bus2IP_Data;
input      [0 : C_SLV_DWIDTH/8-1]         Bus2IP_BE;
input      [0 : C_NUM_REG-1]              Bus2IP_RdCE;
input      [0 : C_NUM_REG-1]              Bus2IP_WrCE;
output     [0 : C_SLV_DWIDTH-1]           IP2Bus_Data;
output                                    IP2Bus_RdAck;
output                                    IP2Bus_WrAck;
output                                    IP2Bus_Error;
output                                    IP2Bus_MstRd_Req;
output                                    IP2Bus_MstWr_Req;
output     [0 : C_MST_AWIDTH-1]           IP2Bus_Mst_Addr;
output     [0 : C_MST_DWIDTH/8-1]         IP2Bus_Mst_BE;
output     [0 : 11]                       IP2Bus_Mst_Length;
output                                    IP2Bus_Mst_Type;
output                                    IP2Bus_Mst_Lock;
output                                    IP2Bus_Mst_Reset;
input                                     Bus2IP_Mst_CmdAck;
input                                     Bus2IP_Mst_Cmplt;
input                                     Bus2IP_Mst_Error;
input                                     Bus2IP_Mst_Rearbitrate;
input                                     Bus2IP_Mst_Cmd_Timeout;
input      [0 : C_MST_DWIDTH-1]           Bus2IP_MstRd_d;
input      [0 : C_MST_DWIDTH/8-1]         Bus2IP_MstRd_rem;
input                                     Bus2IP_MstRd_sof_n;
input                                     Bus2IP_MstRd_eof_n;
input                                     Bus2IP_MstRd_src_rdy_n;
input                                     Bus2IP_MstRd_src_dsc_n;
output                                    IP2Bus_MstRd_dst_rdy_n;
output                                    IP2Bus_MstRd_dst_dsc_n;
output     [0 : C_MST_DWIDTH-1]           IP2Bus_MstWr_d;
output     [0 : C_MST_DWIDTH/8-1]         IP2Bus_MstWr_rem;
output                                    IP2Bus_MstWr_sof_n;
output                                    IP2Bus_MstWr_eof_n;
output                                    IP2Bus_MstWr_src_rdy_n;
output                                    IP2Bus_MstWr_src_dsc_n;
input                                     Bus2IP_MstWr_dst_rdy_n;
input                                     Bus2IP_MstWr_dst_dsc_n;
// -- DO NOT EDIT ABOVE THIS LINE --------------------

  
//----------------------------------------------------------------------------
// Implementation
//----------------------------------------------------------------------------

   //-----------------------------	BUS PROTOCOL STUFF -----------------------------------------------
   
   wire rdy = Bus2IP_MstWr_dst_rdy_n;
   wire ack = Bus2IP_Mst_CmdAck;
   wire com = Bus2IP_Mst_Cmplt; 

   //the RGB values to be written to memory
   wire [7:0] my_read_red;
   wire [7:0] my_read_green;
   wire [7:0] my_read_blue;
   
   //address of memory writes
   reg [31:0] Addr_counter;
   //current line of input
   wire [9:0] line_count; 
   
   //current pixel to be written to memory
   reg [10:0] mycounter; 
	  
   // PLB control signals
   reg REG_IP2Bus_MstWr_Req; 
   reg REG_IP2Bus_MstRd_Req; 
   reg REG_IP2Bus_MstWr_sof_n; 
   reg REG_IP2Bus_MstWr_eof_n;
   reg REG_IP2Bus_Mst_Type; 
   reg REG_IP2Bus_MstRd_dst_dsc_n; 
   reg REG_IP2Bus_MstRd_dst_rdy_n; 
   reg REG_IP2Bus_MstWr_src_dsc_n; 
   reg REG_IP2Bus_MstWr_src_rdy_n; 
   reg [0:31] REG_Bus2IP_MstRd_d; 
   reg [31:0] REG_IP2Bus_Mst_Addr; 
   
	//FSM states
	parameter [5:0] 
		//init_read_states
		str_init = 0, 
		set_init = 1,
		ack_state_init = 2, 
		cmp_init = 3, 
		//burst_write_states
		str = 4,   
		set = 5, 
		ack_state = 6, 
		cmp = 7, 
		st1 = 8;
   
   //fsm
   reg [5:0] y;
   reg [5:0] Y;
   
   //number of lines currently written to the still frame
   reg [12:0] capture_line_count; 
   reg [12:0] y_min, y_max, x_min, x_max; 
   
   //fsm for whether we're writing a still frame or a live video
   parameter [1:0]
		capture_frame = 0, 
		stream_frame = 1; 
		
	reg [1:0] s; 
	reg [1:0] S; 
	

   
  //-----------------------------	BUS PROTOCOL STUFF -----------------------------------------------
  
  
  //----------------------------- VIDEO CAPTURE VERIOLOG -----------------------------------------
	wire RESET_VDEC1_Z = 1'b1;
	wire VDEC1_OE_Z = 1'b0;
	wire VDEC1_PWRDN_Z = 1'b1;

	reg [7:0] R;
	reg [7:0] G;
	reg [7:0] B;
	
	reg [7:0] my_R;
	reg [7:0] my_G;
	reg [7:0] my_B;

	reg [10:0] write_address;
	reg read_enable_lb0;
	reg write_enable_lb0;
	reg crop;
	reg REG_IP2Bus_Mst_Reset;
	wire h_synch_delay;
	wire v_synch_delay;
	wire comp_synch;
	wire blank;
	wire rst;
	wire [9:0] YCrCb_out_422;
	wire Fo_422;
	wire Vo_422;
	wire Ho_422;
	wire Fi_422;
	wire Vi_422;
	wire Hi_422;
	wire Fo_444;
	wire Vo_444;
	wire Ho_444;
	wire ceo_444;
	wire  [9:0] YCrCb_in_422;
	wire  [9:0] Y_out_444;
	wire  [9:0] Cr_out_444;
	wire  [9:0] Cb_out_444;
	wire  [9:0] Y_in_444;
	wire  [9:0] Cr_in_444;
	wire  [9:0] Cb_in_444;
	wire  [7:0] Red;
	wire  [7:0] Green;
	wire  [7:0] Blue;
	wire hblank;
	wire llc_clock_i;
	wire LLC_CLOCK;
	wire clock_27;
	wire clk_27;
	wire clk_13;
	wire clock_13;
	wire clock_40;
	wire clk_40;
	wire system_dcm_rst;
	wire system_dcm_locked;
	wire low = 1'b0;
	wire high = 1'b1;
	wire [10:0] pixel_count;
	wire read_enable_lb1 = !read_enable_lb0;
	wire write_enable_lb1 = ! write_enable_lb0;
	wire [7:0] write_red_data;
	wire [7:0] write_green_data;
	wire [7:0] write_blue_data;
	wire [7:0] read_red_data_lb0;
	wire [7:0] read_green_data_lb0;
	wire [7:0] read_blue_data_lb0;
	wire [7:0] read_red_data_lb1;
	wire [7:0] read_green_data_lb1;
	wire [7:0] read_blue_data_lb1;
	wire one_shot_out;
	wire reset_timing_gen;
	wire [2:0] char_line_count;
	wire [13:0] char_address;
	wire [2:0] char_pixel;
	wire hsync_in;
	wire vsync_in;
	wire blank_in;
	wire comp_synch_in;
	wire hsync_out;
	wire vsync_out;
	wire blank_out;
	wire comp_synch_out;
	wire NTSC_out;
	wire  [9:0] concat_YCrCb_in;

	//coefficient replace
	wire [0:17] green_coef;
	wire [0:17] red_coef;
	wire [0:17] blue_coef;
	
	reg [7:0] delta_red;
	reg [7:0] delta_blue;
	reg [7:0] delta_green;
	reg [9:0] delta;
	
	reg [9:0] blob_counter;
	reg target_detected;
	
	assign Fi_422 = Fo_422;
	assign Vi_422 = Vo_422;
	assign Hi_422 = Ho_422;
	assign YCrCb_in_422[9:0] = YCrCb_out_422[9:0];
	assign  Y_in_444[9:0]  = Y_out_444[9:0];
	assign  Cr_in_444[9:0]  = Cr_out_444[9:0];
	assign  Cb_in_444[9:0]  = Cb_out_444[9:0];

	assign rst = !system_dcm_locked;
	assign write_red_data[7:0] = Red[7:0];
	assign write_green_data[7:0] = Green[7:0];
	assign write_blue_data[7:0] = Blue[7:0];
	assign reset_timing_gen = (rst | one_shot_out);
	assign concat_YCrCb_in = {YCrCb_in[9:2],2'B00};
  
   
   
  //----------------------------------------------------------------------------------------------------------------
  // assign bus signals to FSM controlled registers
  assign IP2Bus_Mst_Type = REG_IP2Bus_Mst_Type;	
  assign IP2Bus_MstRd_Req = REG_IP2Bus_MstRd_Req;	
  assign IP2Bus_MstRd_dst_dsc_n = REG_IP2Bus_MstRd_dst_dsc_n;	
  assign IP2Bus_MstRd_dst_rdy_n = REG_IP2Bus_MstRd_dst_rdy_n;
  assign IP2Bus_MstWr_Req = REG_IP2Bus_MstWr_Req;
  assign IP2Bus_MstWr_sof_n = REG_IP2Bus_MstWr_sof_n;
  assign IP2Bus_MstWr_eof_n = REG_IP2Bus_MstWr_eof_n;
  assign IP2Bus_MstWr_src_rdy_n = REG_IP2Bus_MstWr_src_rdy_n;
  assign IP2Bus_MstWr_src_dsc_n = REG_IP2Bus_MstWr_src_dsc_n;	
 
  assign IP2Bus_Mst_Addr = REG_IP2Bus_Mst_Addr;  
  assign IP2Bus_Mst_Reset = REG_IP2Bus_Mst_Reset;
  
  assign IP2Bus_Mst_Lock = 0;
  
  assign IP2Bus_MstWr_d[0] = 0;
  assign IP2Bus_MstWr_d[1] = 0;
  assign IP2Bus_MstWr_d[2] = 0;
  assign IP2Bus_MstWr_d[3] = 0;
  assign IP2Bus_MstWr_d[4] = 0;
  assign IP2Bus_MstWr_d[5] = 0;
  assign IP2Bus_MstWr_d[6] = 0;
  assign IP2Bus_MstWr_d[7] = 0;
  assign IP2Bus_MstWr_d[8] = my_R[7];
  assign IP2Bus_MstWr_d[9] = my_R[6];
  assign IP2Bus_MstWr_d[10] = my_R[5]; 
  assign IP2Bus_MstWr_d[11] = my_R[4]; 
  assign IP2Bus_MstWr_d[12] = my_R[3]; 
  assign IP2Bus_MstWr_d[13] = my_R[2]; 
  assign IP2Bus_MstWr_d[14] = 0;
  assign IP2Bus_MstWr_d[15] = 0;
  assign IP2Bus_MstWr_d[16] = my_G[7];
  assign IP2Bus_MstWr_d[17] = my_G[6];
  assign IP2Bus_MstWr_d[18] = my_G[5];
  assign IP2Bus_MstWr_d[19] = my_G[4];
  assign IP2Bus_MstWr_d[20] = my_G[3];
  assign IP2Bus_MstWr_d[21] = my_G[2];
  assign IP2Bus_MstWr_d[22] = 0;
  assign IP2Bus_MstWr_d[23] = 0;
  assign IP2Bus_MstWr_d[24] = my_B[7];
  assign IP2Bus_MstWr_d[25] = my_B[6];
  assign IP2Bus_MstWr_d[26] = my_B[5];
  assign IP2Bus_MstWr_d[27] = my_B[4];
  assign IP2Bus_MstWr_d[28] = my_B[3];
  assign IP2Bus_MstWr_d[29] = my_B[2];
  assign IP2Bus_MstWr_d[30] = 0;
  assign IP2Bus_MstWr_d[31] = 0;
    
  //write remainder, irrelevant to the types of requests we're making
  assign IP2Bus_MstWr_rem 		    = 4'b0000; 
  // length of burst, 3200 bytes = 800 words = 800 pixels
  assign IP2Bus_Mst_Length  	    = 12'hc80;   
  // byte enable, makes sure the whole byte is written
  assign IP2Bus_Mst_BE              = 4'b1111;     

  // IAN - these signals are used by the registers
  // irrelevant to our requests
  // assign IP2Bus_Data = 0;              	// IP to Bus data bus
  // assign IP2Bus_RdAck = 0;                // IP to Bus read transfer acknowledgement
  // assign IP2Bus_WrAck  = 0;               // IP to Bus write transfer acknowledgement
  // assign IP2Bus_Error  = 0;               // IP to Bus error response  
  
  
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//USER SW REGISTER STUFF: ADDED BY IAN
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  // --USER nets declarations added here, as needed for user logic

  // Nets for user logic slave model s/w accessible register example
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg0;
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg1;
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg2;
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg3;
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg4;
  wire       [0 : 4]                        slv_reg_write_sel;
  wire       [0 : 4]                        slv_reg_read_sel;
  reg        [0 : C_SLV_DWIDTH-1]           slv_ip2bus_data;
  wire                                      slv_read_ack;
  wire                                      slv_write_ack;
  integer                                   byte_index, bit_index;

  // --USER logic implementation added here

  // ------------------------------------------------------
  // Example code to read/write user logic slave model s/w accessible registers
  // 
  // Note:
  // The example code presented here is to show you one way of reading/writing
  // software accessible registers implemented in the user logic slave model.
  // Each bit of the Bus2IP_WrCE/Bus2IP_RdCE signals is configured to correspond
  // to one software accessible register by the top level template. For example,
  // if you have four 32 bit software accessible registers in the user logic,
  // you are basically operating on the following memory mapped registers:
  // 
  //    Bus2IP_WrCE/Bus2IP_RdCE   Memory Mapped Register
  //                     "1000"   C_BASEADDR + 0x0
  //                     "0100"   C_BASEADDR + 0x4
  //                     "0010"   C_BASEADDR + 0x8
  //                     "0001"   C_BASEADDR + 0xC
  // 
  // ------------------------------------------------------

  assign
    slv_reg_write_sel = Bus2IP_WrCE[0:4],
    slv_reg_read_sel  = Bus2IP_RdCE[0:4],
    slv_write_ack     = Bus2IP_WrCE[0] || Bus2IP_WrCE[1] || Bus2IP_WrCE[2] || Bus2IP_WrCE[3] || Bus2IP_WrCE[4],
    slv_read_ack      = Bus2IP_RdCE[0] || Bus2IP_RdCE[1] || Bus2IP_RdCE[2] || Bus2IP_RdCE[3] || Bus2IP_RdCE[4];

  // implement slave model register(s)
  always @( posedge Bus2IP_Clk )
    begin: SLAVE_REG_WRITE_PROC

      if ( Bus2IP_Reset == 1 )
        begin
          slv_reg0 <= 0;
          slv_reg1 <= 0;
          slv_reg2 <= 0;
          slv_reg3 <= 0;
          slv_reg4 <= 0;
        end
      else if ( line_count == 479 && mycounter == 639 )
		  begin
		    slv_reg0 <= x_min;
          slv_reg1 <= x_max;
          slv_reg2 <= y_min;
          slv_reg3 <= y_max;
		  end
//        case ( slv_reg_write_sel )
//          5'b10000 :
//            for ( byte_index = 0; byte_index <= (C_SLV_DWIDTH/8)-1; byte_index = byte_index+1 )
//              if ( Bus2IP_BE[byte_index] == 1 )
//                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
//                  slv_reg0[bit_index] <= Bus2IP_Data[bit_index];
//          5'b01000 :
//            for ( byte_index = 0; byte_index <= (C_SLV_DWIDTH/8)-1; byte_index = byte_index+1 )
//              if ( Bus2IP_BE[byte_index] == 1 )
//                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
//                  slv_reg1[bit_index] <= Bus2IP_Data[bit_index];
//          5'b00100 :
//            for ( byte_index = 0; byte_index <= (C_SLV_DWIDTH/8)-1; byte_index = byte_index+1 )
//              if ( Bus2IP_BE[byte_index] == 1 )
//                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
//                  slv_reg2[bit_index] <= Bus2IP_Data[bit_index];
//          5'b00010 :
//            for ( byte_index = 0; byte_index <= (C_SLV_DWIDTH/8)-1; byte_index = byte_index+1 )
//              if ( Bus2IP_BE[byte_index] == 1 )
//                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
//                  slv_reg3[bit_index] <= Bus2IP_Data[bit_index];
//          5'b00001 :
//            for ( byte_index = 0; byte_index <= (C_SLV_DWIDTH/8)-1; byte_index = byte_index+1 )
//              if ( Bus2IP_BE[byte_index] == 1 )
//                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
//                  slv_reg4[bit_index] <= Bus2IP_Data[bit_index];
//          default : ;
//        endcase

    end // SLAVE_REG_WRITE_PROC

  // implement slave model register read mux
  always @( slv_reg_read_sel or slv_reg0 or slv_reg1 or slv_reg2 or slv_reg3 or slv_reg4 )
    begin: SLAVE_REG_READ_PROC

      case ( slv_reg_read_sel )
        5'b10000 : slv_ip2bus_data <= slv_reg0;
        5'b01000 : slv_ip2bus_data <= slv_reg1;
        5'b00100 : slv_ip2bus_data <= slv_reg2;
        5'b00010 : slv_ip2bus_data <= slv_reg3;
        5'b00001 : slv_ip2bus_data <= slv_reg4;
			default : slv_ip2bus_data <= 0;
      endcase

    end // SLAVE_REG_READ_PROC

  assign IP2Bus_Data    = slv_ip2bus_data;
  assign IP2Bus_WrAck   = slv_write_ack;
  assign IP2Bus_RdAck   = slv_read_ack;
  assign IP2Bus_Error   = 0;

/////////////////////////////////////////////////////////////////////////////////////////////
// END USER SW REGISTER STUFF: ADDED BY IAN
/////////////////////////////////////////////////////////////////////////////////////////////
  
  //Read in the values to be written to memory
  assign my_read_red[7] = buf_read_data[8]; 
  assign my_read_red[6] = buf_read_data[9]; 
  assign my_read_red[5] = buf_read_data[10]; 
  assign my_read_red[4] = buf_read_data[11]; 
  assign my_read_red[3] = buf_read_data[12]; 
  assign my_read_red[2] = buf_read_data[13]; 
  
  assign my_read_green[7] = buf_read_data[16]; 
  assign my_read_green[6] = buf_read_data[17]; 
  assign my_read_green[5] = buf_read_data[18]; 
  assign my_read_green[4] = buf_read_data[19]; 
  assign my_read_green[3] = buf_read_data[20]; 
  assign my_read_green[2] = buf_read_data[21]; 
  
  
  assign my_read_blue[7] = buf_read_data[24]; 
  assign my_read_blue[6] = buf_read_data[25]; 
  assign my_read_blue[5] = buf_read_data[26]; 
  assign my_read_blue[4] = buf_read_data[27]; 
  assign my_read_blue[3] = buf_read_data[28]; 
  assign my_read_blue[2] = buf_read_data[29]; 
  
	// mem_to_ip control line assignments
	assign buf_address = mycounter; 	// the pixel that we need from the mem_to_ip module
	assign buf_pixel_count = pixel_count; 	//for synchronization - mem_to_ip uses this to know what pixel we're reading from input
	assign buf_line_count = line_count; 		// for synchronization - pixel count uses is to know what line we're reading from input
	assign buf_read_enable = read_enable_lb0; 	// enables a read on buffer 0 in mem_to_ip module
	assign buf_write_enable = write_enable_lb0; 	//enables a write on buffer 0 in mem_to_ip module
	
	//calculate the color replacement coefficients
	assign red_coef = REG_Bus2IP_MstRd_d[16:23] * R[7:0];
	assign green_coef = REG_Bus2IP_MstRd_d[0:7] * G[7:0];
	assign blue_coef = REG_Bus2IP_MstRd_d[8:15] * B[7:0];


/////////////////////////////////////////////////////////////////////////////////////////////////////////
// ADDED BY PATRICK
// detect difference between reference and live image and replace with a basic color for initial testing
/////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	always@(*)
	begin
		if ( R[7:0] > my_read_red[7:0] )
			delta_red = R[7:0] - my_read_red[7:0];
		else
			delta_red = my_read_red[7:0] - R[7:0];
		if ( B[7:0] > my_read_blue[7:0] )
			delta_blue = B[7:0] - my_read_blue[7:0];
		else
			delta_blue = my_read_blue[7:0] - B[7:0];
		if ( G[7:0] > my_read_green[7:0] )
			delta_green = G[7:0] - my_read_green[7:0];
		else
			delta_green = my_read_green[7:0] - G[7:0];
			
		delta = {2'b00,delta_red} + {2'b00,delta_blue} + {2'b00,delta_green};
	end
	
	// target detection logic
	always@(*)
	begin
		if (s == stream_frame && 	                      // only check for target during stream mode
			delta > REG_Bus2IP_MstRd_d[22:31] && 
			blob_counter > REG_Bus2IP_MstRd_d[0:9] &&		 // wait for a certain number of consecutive detections
																		// before flagging target_detected to remove glitches caused
																		// by interlacing
			line_count > 24 &&
			line_count < 456 &&
			mycounter > 40 &&
			mycounter < 624
			)
			target_detected <= 1;
		else
			target_detected <= 0;
	end
	

always@(posedge Bus2IP_Clk)
begin
	if(delta > REG_Bus2IP_MstRd_d[22:31])
			blob_counter <= blob_counter + 10'b0000000001;
	else
			blob_counter <= 10'b0000000000;
end

// track y_min, y_max, x_min, x_max
always@(posedge Bus2IP_Clk)
begin
	if (line_count == 0 && mycounter == 0)
	begin
		y_min <= 0;
		y_max <= 0;
		x_max <= 0;
		x_min <= 640;
	end
	if( target_detected == 1 && y_min == 0)
		y_min <= line_count;
	if (target_detected == 1)
		y_max <= line_count;
	if (target_detected == 1 && mycounter < x_min)
		x_min <= mycounter;
	if (target_detected == 1 && mycounter > x_max)
		x_max <= mycounter;
end

always@(*)
begin
  
  if(s == stream_frame && line_count == y_min && y_min!=0 && mycounter > 620) 	// draw a line at y_min	
  begin
		my_B = 8'h00;
		my_R = 8'h00;
		my_G = 8'hFF;
  end
  else if(s == stream_frame && line_count == (y_max+1) && y_max!=0 && mycounter > 620) 	// draw a line after y_max
  begin
		my_B = 8'h00;
		my_R = 8'hFF;
		my_G = 8'hFF;
  end
  else if(s == stream_frame && mycounter == x_min && x_min != 640 && line_count > 460) 	// draw a line at x_min
  begin
		my_B = 8'h00;
		my_R = 8'h00;
		my_G = 8'hFF;
  end
  else if(s == stream_frame && mycounter == x_max && x_max != 0 && line_count > 460) 	// draw a line at x_max
  begin
		my_B = 8'h00;
		my_R = 8'hFF;
		my_G = 8'hFF;
  end
	else if( target_detected == 1)
	begin
		my_B = 8'hFF;
		my_R = 8'h00;
		my_G = 8'h00;
	end
	else
	begin
		my_B = B; 
		my_G = G; 
		my_R = R; 
	end
end
   
   // FSM for controlling whether we're writing to a still frame or live video
   always @(*)
   begin
		case(s)
		//write a still frame
		capture_frame:
			begin
				//write to a still frame. 4096 is the size of one line in bytes. Multiplying by line_count gives the correct address. 
				// 0x300000 is the still frame offset from our base at 0x90000000. 
				Addr_counter = C_REF_FRAME_ADDR + 4096 * line_count; 
				//write 2000 lines to the still frame, then start streaming
				if (capture_line_count < 4*480 )
					S = capture_frame;
				else
					S = stream_frame; 
			end
		//stream the video data
		stream_frame: 
			begin
				//4096 is the size of one line in bytes. Multiplying by line_count gives the correct address. 80 is used to shift the foreground frame left, to 
				// trim some imperfections
				Addr_counter = C_LIVE_FRAME_ADDR + 4096 * line_count - 80;
				S = stream_frame; 
			end
		default: 
			begin
				Addr_counter = C_REF_FRAME_ADDR + 4096 * line_count - 80; 
				S = capture_frame; 
			end
		endcase
   end
     
   //FSM for controlling memory reads / writes
   always @(*)
   begin
	   case(y)
	   // Do nothing state
	   str_init: 
			begin
				// wait for a new line, then start reading in the control word
				if (pixel_count == 0)
					Y = set_init; 
				else
					Y = str_init; 
			end
	   // request a single beat read, and wait for the bust to acknowledge
	   set_init: 
			begin
				if (Bus2IP_Mst_CmdAck == 1 && Bus2IP_MstRd_sof_n == 0)
					Y = cmp_init; 
				else if (Bus2IP_Mst_CmdAck == 1)
					Y = ack_state_init; 
				else
					Y = set_init; 
			end
	  // wait for the bus to place the data value
	   ack_state_init:
	   		begin
				if (Bus2IP_MstRd_sof_n == 0)
					Y = cmp_init; 
				else
					Y = ack_state_init; 
			end
	   // do nothing state
	   cmp_init: Y = str; 
	   // prepare for burst write
	   str: 			
			begin
					Y = set;
			end
		// request a burst write
		set: 			
			begin
				// wait for an acknowledge signal
				if (rdy == 1)
					Y = set;
				else 
					Y = ack_state;
			end
		//once the acknowledge signal is sent
		ack_state: 			
			begin
				//write 680 pixels
				if (mycounter >= 680)
					Y = cmp;
				else 
					Y = ack_state;
			end
		// complete transaction
		cmp: Y = st1;
		// reset and go back to the start
		st1: Y = str_init;
		default: Y = str_init;
		endcase
   end  	 
   
	
  always @ (posedge Bus2IP_Clk or posedge Bus2IP_Reset)
  begin
	if (Bus2IP_Reset == 1)
		begin 
			//initialize states / signals
			y <= str_init;
			s <= capture_frame; 
			mycounter <= 0; 
			capture_line_count <= 0; 
		end
	else
		begin
			//increment the written pixel count
			if (IP2Bus_MstWr_src_rdy_n == 0 && Bus2IP_MstWr_dst_rdy_n == 0)
			begin
				mycounter <= mycounter + 1; 
			end
			
			//when we're about to start writing a new line, increment line count and reset the mycounter pixel count. 
			if (y == set && Y == ack_state)
			begin
				mycounter <= 0;  
				capture_line_count <= capture_line_count + 1; 
			end

			// latch write data on the bus when the bus is ready
			if (Bus2IP_MstRd_sof_n == 0 || Bus2IP_MstRd_eof_n == 0 || Bus2IP_MstRd_src_rdy_n == 0)
			begin
				REG_Bus2IP_MstRd_d <= Bus2IP_MstRd_d; 
			end
			
			// go to the next state
			y <= Y;
			s <= S; 
		end
  end
  
  // BUS CONTROL SIGNALS
  always@(y)
  begin 
	  case(y)
	  str_init: begin
              REG_IP2Bus_MstWr_sof_n = 1; 
              REG_IP2Bus_MstWr_eof_n = 1;
			  REG_IP2Bus_MstWr_Req = 0; 
			  REG_IP2Bus_Mst_Type = 0; 
			  REG_IP2Bus_Mst_Reset = 0;
			  REG_IP2Bus_MstRd_Req = 0; 
			  REG_IP2Bus_Mst_Addr = 32'h9020d000; //word read address
			  REG_IP2Bus_MstRd_dst_dsc_n = 1; 
			  REG_IP2Bus_MstRd_dst_rdy_n = 1; 
			  REG_IP2Bus_MstWr_src_dsc_n = 1; 
			  REG_IP2Bus_MstWr_src_rdy_n = 1;  
			end
	   set_init: begin
              REG_IP2Bus_MstWr_sof_n = 1; 
              REG_IP2Bus_MstWr_eof_n = 1;
			  REG_IP2Bus_MstWr_Req = 0; 
			  REG_IP2Bus_Mst_Type = 0; 
			  REG_IP2Bus_Mst_Reset = 0;
			  REG_IP2Bus_MstRd_Req = 1; 
			  REG_IP2Bus_Mst_Addr = 32'h9020d000; //word read address
			  REG_IP2Bus_MstRd_dst_dsc_n = 1; 
			  REG_IP2Bus_MstRd_dst_rdy_n = 0; 
			  REG_IP2Bus_MstWr_src_dsc_n = 1; 
			  REG_IP2Bus_MstWr_src_rdy_n = 1;  
			end
	   ack_state_init: begin
              REG_IP2Bus_MstWr_sof_n = 1; 
              REG_IP2Bus_MstWr_eof_n = 1;
			  REG_IP2Bus_MstWr_Req = 0; 
			  REG_IP2Bus_Mst_Type = 0; 
			  REG_IP2Bus_Mst_Reset = 0;
			  REG_IP2Bus_MstRd_Req = 0; 
			  REG_IP2Bus_Mst_Addr = 32'h9020d000; //word read address
			  REG_IP2Bus_MstRd_dst_dsc_n = 1; 
			  REG_IP2Bus_MstRd_dst_rdy_n = 0; 
			  REG_IP2Bus_MstWr_src_dsc_n = 1; 
			  REG_IP2Bus_MstWr_src_rdy_n = 1; 
			end
	   cmp_init: begin
              REG_IP2Bus_MstWr_sof_n = 1; 
              REG_IP2Bus_MstWr_eof_n = 1;
			  REG_IP2Bus_MstWr_Req = 0; 
			  REG_IP2Bus_Mst_Type = 0; 
			  REG_IP2Bus_Mst_Reset = 1;
			  REG_IP2Bus_MstRd_Req = 0; 
			  REG_IP2Bus_Mst_Addr = 32'h9020d000; //word read address
			  REG_IP2Bus_MstRd_dst_dsc_n = 1; 
			  REG_IP2Bus_MstRd_dst_rdy_n = 1; 
			  REG_IP2Bus_MstWr_src_dsc_n = 1; 
			  REG_IP2Bus_MstWr_src_rdy_n = 1; 
			end
	  str: begin
              REG_IP2Bus_MstWr_sof_n = 1; 
              REG_IP2Bus_MstWr_eof_n = 1;
			  REG_IP2Bus_MstWr_Req = 0; 
			  REG_IP2Bus_Mst_Type = 0; 
			  REG_IP2Bus_Mst_Reset = 0;
			  REG_IP2Bus_MstRd_Req = 0; 
			  REG_IP2Bus_Mst_Addr = Addr_counter; 
			  REG_IP2Bus_MstRd_dst_dsc_n = 1; 
			  REG_IP2Bus_MstRd_dst_rdy_n = 1; 
			  REG_IP2Bus_MstWr_src_dsc_n = 1; 
			  REG_IP2Bus_MstWr_src_rdy_n = 1; 
			end
	  set: begin
              REG_IP2Bus_MstWr_sof_n = 0; 
              REG_IP2Bus_MstWr_eof_n = 1;
			  REG_IP2Bus_MstWr_Req = 1; 
			  REG_IP2Bus_Mst_Type = 1; 
			  REG_IP2Bus_Mst_Reset= 0;
			  REG_IP2Bus_MstRd_Req = 0; 
			  REG_IP2Bus_Mst_Addr = Addr_counter; 
			  REG_IP2Bus_MstRd_dst_dsc_n = 1; 
			  REG_IP2Bus_MstRd_dst_rdy_n = 1; 
			  REG_IP2Bus_MstWr_src_dsc_n = 1; 
			  REG_IP2Bus_MstWr_src_rdy_n = 0; 
			end
	  ack_state: begin
              REG_IP2Bus_MstWr_sof_n = 1; 
              REG_IP2Bus_MstWr_eof_n = 1;
			  REG_IP2Bus_MstWr_Req = 1; 
			  REG_IP2Bus_Mst_Type = 1; 
			  REG_IP2Bus_Mst_Reset= 0;
			  REG_IP2Bus_MstRd_Req = 0; 
			  REG_IP2Bus_Mst_Addr = Addr_counter; 
			  REG_IP2Bus_MstRd_dst_dsc_n = 1; 
			  REG_IP2Bus_MstRd_dst_rdy_n = 1; 
			  REG_IP2Bus_MstWr_src_dsc_n = 1; 
			  REG_IP2Bus_MstWr_src_rdy_n = 0;
			end
	  cmp: begin
              REG_IP2Bus_MstWr_sof_n = 1; 
              REG_IP2Bus_MstWr_eof_n = 0;
			  REG_IP2Bus_MstWr_Req = 0; 
			  REG_IP2Bus_Mst_Type = 0; 
			  REG_IP2Bus_Mst_Reset= 1;
			  REG_IP2Bus_MstRd_Req = 0; 
			  REG_IP2Bus_Mst_Addr = Addr_counter; 
			  REG_IP2Bus_MstRd_dst_dsc_n = 1; 
			  REG_IP2Bus_MstRd_dst_rdy_n = 1; 
			  REG_IP2Bus_MstWr_src_dsc_n = 1; 
			  REG_IP2Bus_MstWr_src_rdy_n = 0;
			end	
	  st1: begin
              REG_IP2Bus_MstWr_sof_n = 1; 
              REG_IP2Bus_MstWr_eof_n = 1;
			  REG_IP2Bus_MstWr_Req = 0; 
			  REG_IP2Bus_Mst_Type = 0; 
			  REG_IP2Bus_Mst_Reset= 0;
			  REG_IP2Bus_MstRd_Req = 0; 
			  REG_IP2Bus_Mst_Addr = Addr_counter; 
			  REG_IP2Bus_MstRd_dst_dsc_n = 1; 
			  REG_IP2Bus_MstRd_dst_rdy_n = 1; 
			  REG_IP2Bus_MstWr_src_dsc_n = 1; 
			  REG_IP2Bus_MstWr_src_rdy_n = 1;
			end	
	  default:
		begin
              REG_IP2Bus_MstWr_sof_n = 1; 
              REG_IP2Bus_MstWr_eof_n = 1;
			  REG_IP2Bus_MstWr_Req = 0; 
			  REG_IP2Bus_Mst_Type = 0; 
			  REG_IP2Bus_Mst_Reset= 0;
			  REG_IP2Bus_MstRd_Req = 0; 
			  REG_IP2Bus_Mst_Addr = Addr_counter; 
			  REG_IP2Bus_MstRd_dst_dsc_n = 1; 
			  REG_IP2Bus_MstRd_dst_rdy_n = 1; 
			  REG_IP2Bus_MstWr_src_dsc_n = 1; 
			  REG_IP2Bus_MstWr_src_rdy_n = 1;
			end
	  endcase 
  end

  
  //--------------------------------VIDEO CAPTURE DEFINITIONS ------------------------
  // instantiate the clock input buffers for the line locked clock
	IBUFG LLC_INPUT_BUF (
	//.O  (llc_clock_i),
	.O (clk_27),
	.I  (LLC_CLOCK)
	);

	// instantiate the clock input buffers for the internal clocks
	BUFG CLK_13MHZ_BUF (
	.O  (clk_13),
	.I  (ceo_444)
	);

	// instantiare a DDR output flip flop to center the clock in the pixel data
	OFDDRRSE  PIXEL_CLOCK_DDR_FF(
	.Q   (PIXEL_CLOCK),
	.C0  (clk_27),
	.C1  (!clk_27),
	.CE  (high),
	.D0  (low),
	.D1  (high),
	.R   (reset),
	.S   (low)
	);
									
	// INSTANTIATE THE LINE FIELD DECODER
	// THIS EXTRACTS THE H,V,F BITS FROM THE
	// TIMING REFERENCE CODES
	lf_decode lf_decode_i(
	.rst       (rst),                       // Reset and Clock input
	.clk       (clk_27),                    // 27Mhz for SDTV
	.YCrCb_in  (concat_YCrCb_in),           // data from the input video stream
	.YCrCb_out (YCrCb_out_422),             // data delayed by pipe length
	.NTSC_out  (NTSC_out),                  // high = NTSC format detected
	.Fo        (Fo_422),                    // high = field one (even)
	.Vo        (Vo_422),                    // high = vertical blank
	.Ho        (Ho_422)                     // low = active video
	);

	// CONVERT FROM 4:2:2 DATA TO 4:4:4 DATA
	vp422_444_dup vp422_444_dup_i(
	.rst          (rst),                 // resets input data register and control
	.clk          (clk_27),              // video component rate clock, 27Mhz for SDTV
	.ycrcb_in     (YCrCb_in_422),        // video component data
	.ntsc_in      (NTSC_out),     
	.fi           (Fi_422),              // Low to High signals start of Field One
	.vi           (Vi_422),              // High signals Vertical Blanking
	.hi           (Hi_422),              // High signals Horizontal Blanking
	.ceo          (ceo_444),             // output data rate is 1/2 the clock rate
	.ntsc_out_o   (ntsc_out_o),   
	.fo           (Fo_444),              // Field signal delayed by pipe length
	.vo           (Vo_444),              // Vertical signal delayed by pipe length
	.ho           (Ho_444),              // Horizontal signal delayed by pipe length
	.y_out        (Y_out_444),           // Y out
	.cr_out       (Cr_out_444),          // Cr out
	.cb_out       (Cb_out_444)           // Cb out
	);

	// INSTANTIATE THE COLOR SPACE CONVERTER
	YCrCb2RGB YCrCb2RGB_i(
	.R    (Red),       
	.G    (Green),     
	.B    (Blue),      
	.clk  (clk_13),    
	.rst  (rst),       
	.Y    (Y_in_444),  
	.Cr   (Cr_in_444), 
	.Cb   (Cb_in_444)  
	);  
	

	// INSTANTIATE TWO VIDEO LINE BUFFERS. THESE LINE BUFFERS SWITCH
	// FROM READ TO WRITE AT THE END OF EACH HORIZONTAL BLANKING INTERVAL

	// toggle the line buffer on each Ho_444 (the start of the HBI)
	always @ (posedge Ho_444 or posedge rst) begin
		if (rst) begin
				read_enable_lb0 <= 1'b0;
				write_enable_lb0 <= 1'b1;
				end
		else begin
				read_enable_lb0 <= !read_enable_lb0;
				write_enable_lb0 <= !write_enable_lb0;
				end
		end

	// create the write address counter
	always @ (posedge clk_13 or posedge Ho_444) begin
		if (Ho_444) begin
				write_address <= 11'h000;
				end
		   else begin
				write_address <= write_address + 1;
				end
		end

	// WRITE LINE BUFFERS
	LINE_BUFFER LB0
	(
	.read_clk        (Bus2IP_Clk),
	.read_address    (mycounter),
	.read_enable     (read_enable_lb0),
	.read_red_data   (read_red_data_lb0),
	.read_green_data (read_green_data_lb0),
	.read_blue_data  (read_blue_data_lb0),

	.write_clk        (clk_13),
	.write_address    (write_address - 5), // the -5 is used to align output
	.write_enable     (write_enable_lb0),
	.write_red_data   (write_red_data),
	.write_green_data (write_green_data),
	.write_blue_data  (write_blue_data)
	);

	LINE_BUFFER LB1
	(
	.read_clk         (Bus2IP_Clk),
	.read_address     (mycounter),
	.read_enable      (read_enable_lb1),
	.read_red_data    (read_red_data_lb1),
	.read_green_data  (read_green_data_lb1),
	.read_blue_data   (read_blue_data_lb1),

	.write_clk        (clk_13),
	.write_address    (write_address - 5),  // the -5 is used to align output
	.write_enable     (write_enable_lb1),
	.write_red_data   (write_red_data),
	.write_green_data (write_green_data),
	.write_blue_data  (write_blue_data)
	);

	
	// crop the first and last pixels of the line do to line count differences
	always @ (posedge Bus2IP_Clk or posedge rst) begin
		if (rst) begin
				crop <= 1'b1;
				end
		else if (pixel_count < 5) begin
				crop <= 1'b1;
				end
		else if (pixel_count > 714) begin
				crop <= 1'b1;
				end
		else begin
				crop <= 1'b0;
				end
		end

	// create a mux for the line buffer outputs
	always @ (posedge Bus2IP_Clk or posedge rst) begin
		if (rst ) begin
				R[7:0] <= 8'h00;
				G[7:0] <= 8'h00;
				B[7:0] <= 8'h00;
				end
		else if (read_enable_lb0) begin
				R[7:0] <= read_red_data_lb0[7:0];
				G[7:0] <= read_green_data_lb0[7:0];
				B[7:0] <= read_blue_data_lb0[7:0];
				end
		else begin
				R[7:0] <= read_red_data_lb1[7:0];
				G[7:0] <= read_green_data_lb1[7:0];
				B[7:0] <= read_blue_data_lb1[7:0];
				end

		end

	// INSTANTIATE A ONE SHOT TO DETECT THE FALLING EDGS OF THE FIELD
	// BIT IN THE TIMING REFERENCE CODE. THIS PULSE RESETS THE
	// SVGA TIMING GENERATION MODULE TO SYNCH THE OUTPUT VIDEO WITH
	// THE VIDEO SOURCE
	NEG_EDGE_DETECT NEG_EDGE_DETECT_i
	(
	.clk          (clk_27),     
	.data_in      (Fo_444),     
	.reset        (rst),        
	.one_shot_out (one_shot_out)
	);
		
	// INSTANTIATE THE SVGA TIMING GENERATION MODULE
	SPECIAL_SVGA_TIMING_GENERATION SPECIAL_SVGA_TIMING_GENERATION(
	.pixel_clock          (clk_27),           
	.reset                (reset_timing_gen), 
	//.rst                //(rst),                      // remove the one shot for testing purposes
	.h_synch_delay        (h_synch_delay),    
	.v_synch_delay        (v_synch_delay),    
	.comp_synch           (comp_synch),       
	.blank                (blank),            
	.char_line_count      (char_line_count),  
	.char_address         (char_address),     
	.char_pixel           (char_pixel),       
	.pixel_count          (pixel_count),      
	.line_count_out          (line_count)
	);

	// align the syncs and blank with the video data
	PIPE_LINE_DELAY PIPE_LINE_DELAY
	(
	.clk               (clk_27),       
	.rst               (rst),          
	.hsync_in          (hsync_in),     
	.vsync_in          (vsync_in),     
	.blank_in          (blank_in),     
	.comp_sync_in      (comp_sync_in), 
	.hsync_out         (hsync_out),    
	.vsync_out         (vsync_out),    
	.blank_out         (blank_out),    
	.comp_sync_out     (comp_synch_out)
	);

endmodule

module IBUFG(O, I); // synthesis syn_black_box
output O;
input I;
endmodule

module BUFG(O, I); // synthesis syn_black_box
output O;
input I;
endmodule

module OFDDRRSE (Q, C0, C1, CE, D0, D1, R, S); // synthesis syn_black_box
    output Q;
    input  C0, C1, CE, D0, D1, R, S;
endmodule
